#**** Promise Technology 2017 ****
#			 	 #
# 		 		 #
#	 Install Script		 #
#				 #
#				 #
#********************************#

Product=$1 #ex30, e5x00 a-class, vess, d5x00
NormalOrALUA=$2 #normal, alua
FrontEndType=$3  #fc, iscsi
Action=$4 #install, uninstall

#################################################################  confirm OS version and platform

#check root privilege

root=$(whoami)

if [ "$root" != "root" ]; then
	echo "Please get root privilege or use sudo command to execution."
	exit
fi


#Change all file R/W privilege
(chmod 777 * -fR)

#Operation system, now only care redhat and SUSE
OperationSystem=$(ls /etc/*-release | grep redhat)

#*Get Platform*
Platform=$(uname -i)

if [ "$OperationSystem" != "" ]; then
	OperationSystem=rhel
else
	OperationSystem=suse
fi

if [ "$OperationSystem" == "rhel" ]; then

	Version=$(cat /etc/redhat-release | awk '{print $7}')

else

	Version=$(cat /etc/SuSE-release | awk '{print $3}')

	Version=$(echo $Version | awk '{print $2 "sp" $3}')

    SuseVersion=$(echo $(cat /etc/SuSE-release | awk '{print $3}') | awk '{print $2}')
fi

echo ---- Install Start ----
echo OS Version: $OperationSystem $Version $Platform 


#################################################################


#If install folder exist, means we support this OS, otherwise.

if [ ! -d "$OperationSystem$Version" ] ; then
	echo "###### Sorry we don't support this OS platform currently. ######"
	exit
fi


################################################################# 
#confirm your product could be install or not, now a-class don't support suse11sp1 and rhel6.2

if [ "$OperationSystem" == "rhel" ]; then
	if [ "$Product" == "a-class" ]; then
		if [ "$Version" == "6.2" ]; then	
			echo "Sorry, we don't support your product in this system"
			exit
		fi
	fi
else
	if [ "$Product" == "a-class" ]; then
		if [ "$Version" == "11sp1" ]; then	
			echo "Sorry, we don't support your product in this system"
			exit		
		fi
	fi
fi

#################################################################

if [ "$OperationSystem" == "suse" ] && [ "$Version" == "11sp3" ] && [ "$NormalOrALUA" == "alua" ]; then
	KernelVersion=$(uname -r)
	MinorVersion1=$(echo $KernelVersion | cut  -d '.' -f 3)
	MinorVersion1=$(echo $MinorVersion1 | cut  -d '-' -f 1)  	
	MinorVersion2=$(echo $KernelVersion | cut  -d '.' -f 4)
	MinorVersion2=$(echo $MinorVersion2 | cut  -d '-' -f 1) 
	#echo Kernel Minor Version : $MinorVersion1.$MinorVersion2 
	
	if [  "$MinorVersion1" -lt 101 ]; then
		echo "Your kernel :"$KernelVersion", please upgrade your current kernel to 3.0.101-0.8 or above version.(exclude 3.0.101-0.15)"
		exit
	else
		if [  "$MinorVersion1" == "101" ] && [  "$MinorVersion2" == "15" ]; then 
			echo "Your kernel :"$KernelVersion", please upgrade your current kernel to 3.0.101-0.21.1 or above version."
			exit
		fi
	fi
fi

#################################################################


if [ "$Action" == "install" ]; then	

	if [ "$NormalOrALUA" == "alua" ]; then

		#copy libpriopromise_alua.so	

		if [ -d "$OperationSystem$Version/$Platform/alua/patch" ]; then  

			echo "Copy ALUA patch file ...(Don't need care if show omitting folders)"		
			
			if [ "$Platform" == "x86_64" ]; then
			#	echo "copy libpriopromise_alua.so ..."
				cp -i $OperationSystem$Version/$Platform/alua/patch/* /lib64/multipath/			
			else
			#	echo "copy libpriopromise_alua.so ..."
				cp -i $OperationSystem$Version/$Platform/alua/patch/* /lib/multipath/			
			fi
				
		fi

		#Below behavior only affect at SLES11 sp1
		if [ "$OperationSystem" == "suse" ] && [ "$Version" == "11sp1" ]; then

			#get kernel version to see if need upgrade alua module
			KernelVersion=$(uname -r)
			MinorVersion=$(echo $KernelVersion | cut  -d '.' -f 4)
			MinorVersion=$(echo $MinorVersion | cut  -d '-' -f 1)  
			echo Kernel Minor Version : $MinorVersion
			if [ "$MinorVersion" -lt 27 ]; then
				echo "Your kernel version($KernelVersion) was older than 2.6.32.27,"
				read -p "Do you want to upgrade your ALUA modules? (Y/N):" yn
				
				if [ "$yn" == "y" ] || [ "$yn" == "Y" ]; then	
					#copy org alua module
					cp /lib/modules/$(uname -r)/kernel/drivers/scsi/device_handler/scsi_dh_alua.ko /lib/modules/$(uname -r)/kernel/drivers/scsi/device_handler/scsi_dh_alua.org.ko			
					#force install alua module			
					rpm -ivh $OperationSystem$Version/$Platform/alua/patch/alua_module/scsi_dh_alua_sles11sp1-2-1.$Platform.rpm --force
				fi
			else
				echo "Don't Need Patch ALUA module"
			fi 		
		fi	
	fi

	echo "Copy MPIO configuration file ..."

	#copy multipath.conf
    if [ "$NormalOrALUA" == "alua" ]; then
        if [ "$Product" == "d5x00" ]; then
            cp -i $OperationSystem$Version/$Platform/alua/multipathD5x00.conf.$FrontEndType /etc/multipath.conf
        else
            cp -i $OperationSystem$Version/$Platform/alua/multipath.conf.$FrontEndType /etc/multipath.conf
        fi
    else
        cp -i $OperationSystem$Version/$Platform/normal/multipath.conf.$FrontEndType /etc/multipath.conf
    fi

	#install SUSEsp2 multipath-tools fix if need
	if [ "$OperationSystem" == "suse" ] && [ "$Version" == "11sp2" ]; then	
	
			echo "We will upgrade your multipath-tools version to 0.4.9-0.68.1"
			echo "(##NOTICE##) If your multipath-tools is newer than this version(0.4.9-0.68.1) or" 
			echo "you don't want to do this, please press (N)"
			read -p "Do you want to upgrade your multipath-tools? (Y/N):" yn

			if [ "$yn" == "y" ] || [ "$yn" == "Y" ]; then
				rpm -ivh $OperationSystem$Version/$Platform/alua/patch/multipath-tools-0.4.9-0.68.1.$Platform.rpm --force
			fi
	fi			

	if [ "$FrontEndType" == "iscsi" ]; then
		
		read -p "Do you agree we replace your iscsid.conf to PROMISE configuration?(Y/N):" yn1

		if [ "$yn1" == "y" ] || [ "$yn1" == "Y" ]; then

			echo "Remove all iSCSI device record."
			if [ "$OperationSystem" == "rhel" ]; then
				rm /var/lib/iscsi/nodes/* -fR 
			else
				rm /etc/iscsi/nodes/* -fR 
			fi
			
			echo "Logout all iSCSI sessions."
			(iscsiadm -m node -u all)
			cp -i iscsi/iscsid.conf.$OperationSystem /etc/iscsi/iscsid.conf
			echo "Copy iscsid.conf ..."

			if [ "$OperationSystem" == "rhel" ]; then
				(service iscsi restart)
				(service iscsid restart)
			else
				(service open-iscsi restart)
			fi	
			
			echo "Please rediscovery your target device information."
		fi		

	fi

	echo "Enable multipath service ..."
	
	read -p "Do you want to start your multipath service? (Y/N):" yn

	if [ "$yn" == "y" ] || [ "$yn" == "Y" ]; then
		if [ "$OperationSystem" == "suse" ]; then
			(chkconfig multipathd on)
			(service multipathd stop)
			(multipath -F)
			(multipath)
			(service multipathd start)

		elif [ "$OperationSystem" == "rhel" ]; then
			(mpathconf --enable)
			(service multipathd stop)
			(multipath -F)
			(multipath)
			(service multipathd start)
		fi
	fi

	if [ "$OperationSystem" == "suse" ] && [ "$SuseVersion" -ge 12 ]; then
        WWID=$(multipath -v2 | awk '{print $5}' | cut  -d ':' -f 1)
        if [ "$WWID" != "" ]; then
            declare -i Number=$(grep -n 'blacklist {' /etc/multipath.conf | cut  -d ':' -f 1)
            (sed -i ""$Number"a wwid "$WWID"" /etc/multipath.conf)
        fi
	fi
    
    if [ -a /proc/sys/fs/aio-max-nr ]; then
        Aiomaxnr=$(cat /proc/sys/fs/aio-max-nr)
        if [ "$Aiomaxnr" -lt 1048576 ]; then
            if [ -a /etc/sysctl.conf ]; then
                sed -i '/fs.aio-max-nr/d' /etc/sysctl.conf
                echo 'fs.aio-max-nr = 1048576' >> /etc/sysctl.conf
                (sysctl -p /etc/sysctl.conf)
            fi
        fi
    fi
    
	echo ""
	echo ""
	echo "Install Promise Linux MPIO Done, change will TAKE EFFECT after REBOOT."
	echo ""
	echo ""

	read -p "Reboot your system now?(Y/N)" yn2
	
	if [ "$yn2" == "y" ] || [ "$yn2" == "Y" ]; then
		(reboot)			
	fi

#################################################################

elif [ "$Action" == "uninstall" ]; then

    if [ "$OperationSystem" == "suse" ] && [ "$SuseVersion" -ge 12 ]; then
        (chkconfig multipathd off)
    fi

	Platform=$(uname -i)

	if [ "$Platform" == "x86_64" ]; then

		# check libpromise_alua exist or not
		if [ -a /lib64/multipath/libpriopromise_alua.so ]; then 
			rm --interactive /lib64/multipath/libpriopromise_alua.so
		fi		

		# check libpromise_tur exist or not
		if [ -a /lib64/multipath/libcheckpromise_tur.so ]; then 		
			rm --interactive /lib64/multipath/libcheckpromise_tur.so
		fi

		# check multipath conf exist or not
		if [ -a /etc/multipath.conf ]; then 	
			rm --interactive /etc/multipath.conf
		fi
	
		# remove alua rpm if exist
		PackExist=$(rpm -qa | grep scsi_dh_alua_sles11sp1-2-1)
		if [ "$PackExist" != "" ]; then
			rpm -e scsi_dh_alua_sles11sp1-2-1		
		fi

		#retrieve orig alua module if exist
		if [ -a /lib/modules/$(uname -r)/kernel/drivers/scsi/device_handler/scsi_dh_alua.org.ko ]; then
			mv /lib/modules/$(uname -r)/kernel/drivers/scsi/device_handler/scsi_dh_alua.org.ko /lib/modules/$(uname -r)/kernel/drivers/scsi/device_handler/scsi_dh_alua.ko	
		fi
	

	else

		# check libpromise_alua exist or not
		if [ -a /lib/multipath/libpriopromise_alua.so ]; then 
			rm --interactive /lib/multipath/libpriopromise_alua.so
		fi		

		# check libpromise_tur exist or not
		if [ -a /lib/multipath/libcheckpromise_tur.so ]; then 		
			rm --interactive /lib/multipath/libcheckpromise_tur.so
		fi

		# check multipath conf exist or not
		if [ -a /etc/multipath.conf ]; then 	
			rm --interactive /etc/multipath.conf
		fi

		# remove alua rpm if exist
		PackExist=$(rpm -qa | grep scsi_dh_alua_sles11sp1-2-1)
		if [ "$PackExist" != "" ]; then
			rpm -e scsi_dh_alua_sles11sp1-2-1		
		fi

		#retrieve orig alua module if exist
		if [ -a /lib/modules/$(uname -r)/kernel/drivers/scsi/device_handler/scsi_dh_alua.org.ko ]; then
			mv /lib/modules/$(uname -r)/kernel/drivers/scsi/device_handler/scsi_dh_alua.org.ko /lib/modules/$(uname -r)/kernel/drivers/scsi/device_handler/scsi_dh_alua.ko	
		fi
					
	fi

    if [ "$OperationSystem" == "suse" ] && [ "$SuseVersion" -ge 12 ]; then
        echo ""
        echo ""
        echo "Uninstall Promise Linux MPIO Done."
        echo ""
        echo ""

        read -p "Reboot your system now?(Y/N)" yn2

        if [ "$yn2" == "y" ] || [ "$yn2" == "Y" ]; then
            (reboot)
        fi
    fi

else
	echo "Please Input Correct Parameters. ( ./promise.sh ex30/e5x00/a-class/vess/d5x00 normal/alua fc/iscsi install/uninstall)"
fi



